//
//  BibDatabase.h, Abstract class for AllDatabases,
//						LocalDatabase, RemoteDatabase, SmartLibrary, ManualLibrary
//  BibCompanion
//
//  Created by Hitoshi Nakahara on 04/11/15.
//  Copyright 2004 Hit-C Soft House. All rights reserved.
//

#import <Cocoa/Cocoa.h>

@class	BibDatabase, RealDatabase, DatabaseGroup;
@class	BibItem, BCMainController, DBListController, AuthorName;

#ifndef	BCDatabaseKind
typedef	enum BCDatabaseKind {
	iBibDatabaseKind_Undefined = -1,
	iBibDatabaseKind_Root = 0,				// root node
	iBibDatabaseKind_DbGroup,				// database group
	iBibDatabaseKind_LibGroup,				// library group
	iBibDatabaseKind_SmartLib,				// smart library
	iBibDatabaseKind_ManuLib,				// manual library
	iBibDatabaseKind_Local,					// local database
	iBibDatabaseKind_Remote					// remote database (not supported)
} BCDatabaseKind;

typedef enum BC_WaitingStatus {
	BC_WaitingForProceed_Proceed = 0,
	BC_WaitingForProceed_Asking,
	BC_WaitingForProceed_Canceled
} BC_WaitingStatus;


#define		BC_DBTree_PropertyKey		@"Database Tree Property"
#define		BC_DBTree_ChildrenKey		@"Database Tree Children"

#define		BC_DBTree_Property_Kind			@"kind"
#define		BC_DBTree_Property_UniqueID		@"unique ID"
#define		BC_DBTree_Property_Description	@"description"
#define		BC_DBTree_Property_Expanded		@"expanded"

#define		BC_SaveOptionsKey_SaveAs		@"Save Options Save a Copy"
#define		BC_SaveOptionsKey_LF			@"Save Options Line Feed"
#define		BC_SaveOptionsKey_Encoding		@"Save Options Encoding"
#define		BC_SaveOptionsKey_Attachments	@"Save Options Attachments"
#define		BC_SaveOptionsKey_Description	@"Save Options Description"

#endif

@interface DatabaseProperties : NSObject
{
	NSString			*uniqueID;			// unique ID for the database
	NSString			*description;		// nominal name for the database
	NSImage				*icon;				// icon for the database
	BOOL				groupExpanded;		// true if it is a group and expanded

	BibDatabase			*database;			// owner
}

- (id)initWithDatabase:(BibDatabase*)db;
- (void)readFromDictionary:(NSDictionary*)propertyDic;
- (void)makeCopyOf:(DatabaseProperties*)source;
- (NSMutableDictionary*)makePropertyDictionary;

- (BCDatabaseKind)kind;

- (BibDatabase*)database;
- (void)setDatabase:(BibDatabase*)db;
- (NSString*)description;
- (void)setDescription:(NSString*)aDescription;
- (NSImage*)icon;
- (void)setIcon:(NSString*)iconName;
- (BOOL)groupExpanded;
- (void)setExpanded:(BOOL)expand;

- (NSString*)uniqueID;
- (void)setUniqueID:(NSString*)idNo;
- (void)generateUniqueID;

- (void)propertiesChanged;

@end

//////////

@interface BibDatabase : NSObject
{
	BCDatabaseKind		kind;
	DatabaseProperties	*properties;

    BibDatabase			*nodeParent;
    NSMutableArray		*nodeChildren;

	BCMainController	*mainDocument;
	DBListController	*controllerDbList;

	NSMutableArray		*itemList;		// array of BibItem
	NSMutableDictionary	*authorList;	// dict of name -> NSArray of AuthorName+BibItem's

	BOOL				nowLoading;
	BOOL				stopLoading;
	BC_WaitingStatus	waitingStatus;
	int					numLoading;
	
	BOOL				needsUpdate;
	BOOL				dirty;
}

+ (BibDatabase*)createRootNodeWithDictionary:(NSDictionary*)treeDic;

- (id)init;
- (id)initForRootWithDictionary:(NSDictionary*)treeDic;
- (id)initWithParent:(BibDatabase*)parent children:(NSArray*)children;

- (void)dealloc;

- (NSDictionary*)makeTreeDictionary;

- (BCDatabaseKind)kind;
- (NSString*)uniqueID;
- (NSString*)description;
- (NSImage*)icon;

- (void)openProperty;
- (void)propertiesChanged;

- (NSAttributedString*)attributedName;

//- (BOOL)isEnabled;
//- (BOOL)isEditable;
//- (BOOL)canReadFile;
//- (BOOL)canWriteFile;
//- (void)setAttributedName:(NSAttributedString*)name;
- (int)numItems;
- (int)numAuthors;
- (NSArray*)allItems;
- (NSDictionary*)allAuthors;

- (BOOL)containsItem:(BibItem*)item;
- (BOOL)childContainsItem:(BibItem*)item;

//- (BOOL)prepareForRemove;

- (void)makeAuthorList;

- (BOOL)appendBibItem:(BibItem*)bibItem;
- (BOOL)removeBibItem:(BibItem*)bibItem removeFile:(BOOL)removeFile;
- (int)removeBibItems:(NSArray*)bibItems removeFile:(BOOL)removeFile;
- (void)removeItemFromLibraries:(BibItem*)bibItem;

- (void)addAuthorsForItem:(BibItem*)bibItem;
- (void)replaceAuthorsForItem:(BibItem*)bibItem;
- (void)removeAuthorsForItem:(BibItem*)bibItem;

- (void)addAuthor:(AuthorName*)name forBibItem:(BibItem*)bibItem;
- (void)removeAuthor:(AuthorName*)name forBibItem:(BibItem*)bibItem;

- (NSComparisonResult)compare:(BibDatabase*)anotherDB;

- (void)startLoading;
- (void)finishLoading;
- (void)setStopLoading;
- (void)setWaitingStatus:(BC_WaitingStatus)status;
- (BOOL)isLoading;
- (BC_WaitingStatus)waitingStatus;
- (void)cleanForCancelLoading;

- (NSDictionary*)fieldNameList;

- (BOOL)isDirty;
- (void)setDirty:(BOOL)edited;

- (NSArray*)readingErrors;

- (BOOL)saveManualLibs;
- (BOOL)saveDatabase;
- (BOOL)saveAsDatabase:(NSString*)databaseName options:(NSDictionary*)options;

- (NSMutableDictionary*)defaultSaveOptions;
- (NSData*)makeContentsDataWithOptions:(NSDictionary*)saveOptions;
- (NSString*)databaseBibItemsStringWithOptions:(NSDictionary*)saveOptions;

+ (void)fileOperationError:(NSString*)error forFile:(NSString*)path;

- (void)prefsChanged;

- (void)getNumLocal:(int*)numLocal andNumRemote:(int*)numRemote andNumSmart:(int*)numSmart andNumManual:(int*)numManual;
- (BibDatabase*)findDatabaseID:(NSString*)dbID withKind:(BCDatabaseKind)dbKind;

- (void)setNodeParent:(BibDatabase*)parent;
- (BibDatabase*)nodeParent;

- (void)insertChild:(BibDatabase*)child atIndex:(int)index;
- (void)insertChildren:(NSArray*)children atIndex:(int)index;
- (void)removeChild:(BibDatabase*)child;
- (void)removeFromParent;
- (void)_removeChildrenIdenticalTo:(NSArray*)children;

- (int)indexOfChild:(BibDatabase*)child;
- (int)indexOfChildIdenticalTo:(BibDatabase*)child;

- (int)numberOfChildren;
- (NSArray*)children;
- (BibDatabase*)firstChild;
- (BibDatabase*)lastChild;
- (BibDatabase*)childAtIndex:(int)index;

- (BOOL)isDescendantOfNode:(BibDatabase*)node;
- (BOOL)isDescendantOfNodeInArray:(NSArray*)nodes;

- (BOOL)isLeaf;
- (BOOL)isGroup;
- (BOOL)groupExpanded;
- (void)setExpanded:(BOOL)expand;
- (void)setExpansionStateWith:(NSOutlineView*)outlineView;

- (BOOL)isDatabaseNode;
- (BOOL)isLibraryGroup;
- (BOOL)isLocalDatabase;
- (BOOL)isRealDatabase;
- (BOOL)isLibrary;
- (BOOL)isManualLib;
- (BOOL)isSmartLib;

- (BOOL)containsManualSmartLib;
- (void)updateSmartLibrary:(DatabaseGroup*)databases;

- (void)resetNeedsUpdate;
- (BOOL)needsUpdate;

- (int)removeBibItems:(NSArray*)bibItems removeFile:(BOOL)removeFile;
- (void)bibItemRemoved:(BibItem*)item;

- (BOOL)bibItemChanged:(BibItem*)item;

- (void)databaseOpened:(RealDatabase*)realDB;
- (void)databaseClosed:(RealDatabase*)realDB;

- (BibItem*)addCopyOfBibItem:(BibItem*)bibItem;
- (void)appendNewItemForSmartLib:(BibItem*)bibItem;
- (void)replaceAuthorsForLib:(BibItem*)bibItem;

- (void)removeAllChildren;

- (void)rebuildBibItemList:(BOOL)propagate;
- (BOOL)removeLibDataFile;

- (NSMutableArray*)getManualLibFiles;
- (NSArray*)getManualLibraries;

- (void)move:(BibDatabase*)item toIndex:(int)newIndex;

@end
